import logging
import os
from glue2protojson import utils
from glue2protojson import gluetypes


if __name__ == '__main__':
    logging.basicConfig(
        format='%(asctime)s:::%(levelname)s:::%(name)s:::%(module)s:::%(funcName)s:::%(message)s',
        level=logging.INFO)
    logger = logging.getLogger(__name__)

    # Read Schema from Glue
    glue_schema_tenant = utils.get_schema_from_glue(database=os.environ['GLUE_DATABASE'],
                                                    table=os.environ['GLUE_TABLE'])
    logger.info(f'GLUE CONTENT:::{glue_schema_tenant}')

    # Parse Glue Schema to Json
    glue_table = utils.get_glue_table(glue_schema=glue_schema_tenant)
    logger.info(f'GLUE CONTENT PARSED TO JSON:::{glue_table}')

    # Glue Schema to Proto
    proto_text = utils.get_proto_text(schema=glue_table, base_name='Tenant')
    logger.info(f'GLUE CONTENT PARSED TO PROTO:::{proto_text}')



from typing import Type, List, Mapping


class GlueColumn:
    def __init__(self, col_type: Type, proto_type: str):
        self.col_type: Type = col_type
        self.proto_type: str = proto_type

    def __str__(self):
        return self.proto_type

    def __repr__(self):
        return self.__str__()


class GlueColumnString(GlueColumn):
    def __init__(self):
        super().__init__(col_type=str, proto_type='string')


class GlueColumnBoolean(GlueColumn):
    def __init__(self):
        super().__init__(col_type=bool, proto_type='bool')


class GlueColumnStruct(GlueColumn):
    def __init__(self, content: Mapping[str, object], content_name: str):
        super().__init__(col_type=Mapping[str, object], proto_type='message')
        self.content = content
        self.content_name = ''.join([w.capitalize() for w in content_name.lower().split('_')])

    def __str__(self):
        return f'{self.content}'


class GlueColumnArray(GlueColumn):
    def __init__(self, content: [GlueColumn, GlueColumnStruct], content_name: str):
        super().__init__(col_type=List[type(content)], proto_type='repeated')
        self.content = content
        self.content_name = content_name

    def __str__(self):
        if type(self.content) == GlueColumnStruct:
            return f'{self.proto_type} {self.content.content_name}'
        else:
            return f'{self.proto_type} {self.content}'



from distutils.core import setup
setup(
  name='glue2protojson',
  packages=['glue2protojson'],
  version='0.0.1',
  license='MIT',
  description='Get a AWS Glue table schema and convert it to JSON of meta types and proto messages definition',
  author='Ricardo Edo Nevado',
  author_email='ricardo.edo@gmail.com',
  url='https://github.com/ricardoedo',
  download_url='https://github.com/ricardoedo/glue2protojson/archive/v0.0.1-alpha.tar.gz',
  keywords=['AWS', 'GLUE', 'GRPC', 'PROTO', 'PROTOBUF'],
  install_requires=[            # I get to this in a second
          'boto3>=1.10.25',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.6',
  ],
)
